@@ -1,6 +1,5 @@
-from collections import OrderedDict
 from dis import Bytecode, opmap
-from functools import lru_cache
+from operator import is_, not_
 from types import CodeType, FunctionType
 
 from lazy._thunk import thunk, strict
@@ -42,17 +41,38 @@ def _default(opcode, arg):
         yield arg.to_bytes(2, 'little')
 
 
-class LazyConverter(object):
-    class Indexer(object):
-        def __init__(self, converter, key):
-            self._converter = converter
-            self._key = key
+def _lazy_is(a, b, *, is_=is_):
+    """
+    Used instead of the `is` operator.
+    """
+    return thunk(is_, a, b)
 
-        @lru_cache(1)
-        def __call__(self):
-            return self._converter._consts.index(self._key).to_bytes(
-                2, 'little',
-            )
+
+def _lazy_not(a, *, not_=not_):
+    """
+    Used instead of the `not` operator.
+    """
+    return thunk(not_, a)
+
+
+class LazyConverter(object):
+    class Consts(object):
+        def __init__(self):
+            self._idx = 0
+            self._obs = {}
+
+        def index(self, ob):
+            try:
+                return self._obs[ob]
+            except KeyError:
+                self._obs[ob] = idx = self._idx.to_bytes(2, 'little')
+                self._idx += 1
+                return idx
+
+        @property
+        def contents(self):
+            obs = self._obs
+            return tuple(sorted(obs, key=lambda e: obs[e]))
 
     class LazyTransformations(dict):
         def __new__(cls, converter):
@@ -72,23 +92,18 @@ class LazyConverter(object):
     def __init__(self, f, _globals=None):
         self.f = f
         self.code = _getcode(f)
-        self._names = OrderedDict()
-        self._thunk_idx = self.Indexer(self, thunk)
-        self._strict_idx = self.Indexer(self, strict)
-        self._fromvalue_idx = self.Indexer(self, thunk.fromvalue)
+
+        # Construct the const objects.
+        self._consts = self.Consts()
+        for c in self.code.co_consts + (strict, thunk.fromvalue):
+            self._consts.index(c)
+
         self.transformations = self.LazyTransformations(self)
         self._globals = _globals if _globals is not None else f.__globals__
         self._call_args_idx = None
         self._call_kwargs_idx = None
         self._co_total_argcount = None
 
-    @property
-    @lru_cache(1)
-    def _consts(self):
-        return tuple(
-            _const_convert(c, self._globals) for c in self.code.co_consts
-        ) + (thunk, strict, thunk.fromvalue)
-
     @property
     def converted_function(self):
         f = self.f
@@ -118,10 +133,7 @@ class LazyConverter(object):
         self._call_args_idx = (len_co_varnames - 2).to_bytes(2, 'little')
         self._co_total_argcount = co.co_argcount + co.co_kwonlyargcount
 
-        bc = b''.join(
-            b() if isinstance(b, self.Indexer) else b
-            for b in self._lazy_bytecode
-        )
+        bc = b''.join(self._lazy_bytecode)
 
         return CodeType(
             co.co_argcount,
@@ -130,7 +142,7 @@ class LazyConverter(object):
             co.co_stacksize + 1,
             co.co_flags,
             bc,
-            self._consts,
+            self._consts.contents,
             co.co_names,
             co_varnames,
             co.co_filename,
@@ -150,8 +162,11 @@ class LazyConverter(object):
             yield from self.transformations[b.opname](b.opcode, b.arg)
 
     def transform_MAKE_FUNCTION(self, opcode, arg):
+        """
+        Functions should have strict names.
+        """
         yield ops.LOAD_CONST
-        yield self._strict_idx
+        yield self._consts.index(strict)
         yield ops.ROT_TWO
         yield ops.CALL_FUNCTION
         yield b'\x01\x00'
@@ -161,8 +176,11 @@ class LazyConverter(object):
     transform_MAKE_CLOSURE = transform_MAKE_FUNCTION
 
     def _transform_name(self, opcode, arg):
+        """
+        Loading a name immediatly wraps it in a `thunk`.
+        """
         yield ops.LOAD_CONST
-        yield self._fromvalue_idx
+        yield self._consts.index(thunk.fromvalue)
         yield bytes((opcode,))
         yield arg.to_bytes(2, 'little')
         yield ops.CALL_FUNCTION
@@ -171,11 +189,41 @@ class LazyConverter(object):
     transform_LOAD_NAME = transform_LOAD_GLOBAL = _transform_name
 
     def transform_LOAD_FAST(self, opcode, arg):
+        """
+        Wrap arg lookups in thunks to be safe.
+        """
         if arg > self._co_total_argcount:
             yield from _default(opcode, arg)
         else:
             yield from self._transform_name(opcode, arg)
 
+    def transform_COMPARE_OP(self, opcode, arg):
+        """
+        Replace the `is` operator to act on the values the thunks represent.
+        This makes `is` lazy.
+        """
+        if arg != 8:  # is
+            yield from _default(opcode, arg)
+            return
+
+        yield ops.LOAD_CONST
+        yield self._consts.index(_lazy_is)
+        yield ops.ROT_THREE
+        yield ops.CALL_FUNCTION
+        yield b'\x02\x00'
+
+    def transform_UNARY_NOT(self, opcode, arg):
+        """
+        Replace the `not` operator to act on the values that the thunks
+        represent.
+        This makes `not` lazy.
+        """
+        yield ops.LOAD_CONST
+        yield self._consts.index(_lazy_not)
+        yield ops.ROT_TWO
+        yield ops.CALL_FUNCTION
+        yield b'\x01\x00'
+
 
 def lazy_function(f):
     """
